/*
    BNoise - A 2D noise generator.
    Copyright (C) 2023 Pascal Baillehache info@baillehachepascal.dev
    https://baillehachepascal.dev
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef BNOISE_H
#define BNOISE_H
#include <stdlib.h>
#include <string.h>
#include <math.h>

// BNoise structure
typedef struct {

  // Dimension of the grid (square) in number of vertices
  int dim;
  float fDim;

  // Noise data
  float* map;
  float* curve[2][2];
} BNoise;

// Allocate memory and create a BNoise instance
// Inputs:
//   order: order of the noise. The grid will be of size (dim+1)*(dim+1) in
//        number of vertices and the Bezier curve will be of order (4+dim).
// Output:
//   Return the new BNoise instance
BNoise* BNoiseAlloc(int const order);

// Free memory used by a BNoise
// Inputs:
//   that: the BNoise
void BNoiseFree(BNoise** const that);

// Get the noise value at a given position
// Inputs:
//   that: the BNoise
//   pos: the 2D position (one tile is contained in [0,1]x[0,1])
// Output:
//   Return the noise value at the position, in [0,1]
float BNoiseGet(
  BNoise* const that,
    float const pos[2]);
#endif
